//**************************************************************************************************
// main.cpp
// This app demonstrates a scrolling background
// Created by: Keith Ditchburn 2005
//**************************************************************************************************

// Exclude rarely-used stuff from Windows headers
#define VC_EXTRALEAN		

// Windows Header Files:
#include <windows.h>

// Resource header file
#include "resource.h"

// DirectX header files
#include <d3d9.h>
#include <d3dx9.h>

// Function prototypes
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
void Render();
bool InitialiseDirectX(HWND hWnd);
void CleanupDirectX();
bool CreateSprite();

// DirectX globals
LPDIRECT3D9 gD3dObject=NULL;		// the directX object
LPDIRECT3DDEVICE9 gD3dDevice=NULL;	// the directX device
LPD3DXSPRITE gSprite=NULL;			// our sprite interfaces
LPDIRECT3DTEXTURE9 gTexture=NULL;	// our texture

// Texture offset
int gOffset=0;

// Sizes
int gScreenWidth=0;
int gScreenHeight=0;
int gTextureWidth=0;
int gTextureHeight=0;

/**************************************************************************************************
Desc: This is the main entry point of the program. 
**************************************************************************************************/
int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
	// Register our window class
	WNDCLASSEX wcex;
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= 0;	
	wcex.lpfnWndProc	= (WNDPROC)WndProc;																
	wcex.cbClsExtra		= 0;												
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_ICON1);	
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);					
	wcex.hbrBackground	= 0;						
	wcex.lpszMenuName	= (LPCSTR)IDC_OURMENU;							
	wcex.lpszClassName	= "OurWindowClass";							
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_ICON1);

	if (RegisterClassEx(&wcex)==0)
		return 0;

	// Create the window, note I want a 256 by 400 screen area so I adjust by 8 and 58 to take into acount the border etc.
	// There are better ways of doing this.....
	HWND hWnd = CreateWindow("OurWindowClass", "Scrolling Background", WS_OVERLAPPEDWINDOW,
	  CW_USEDEFAULT,CW_USEDEFAULT, 256+8, 400+58, NULL, NULL, hInstance, NULL);

	if (hWnd==NULL)
		return 0;

	// Obtain the screen rect
	RECT screenRect;
	GetClientRect(hWnd,&screenRect);

	gScreenWidth=screenRect.right;
	gScreenHeight=screenRect.bottom;

	// Before displaying our window for the first time create the D3D device
	if (!InitialiseDirectX(hWnd))
		return 0;

	if (!CreateSprite())
	{
		CleanupDirectX();
		return -1;
	}

	// Display the window and force an initial paint
	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	// Enter our program loop - this loop continues until the program exits
	// For maximum performance whenever the application is idle (not getting messages) render our scene
    MSG msg;
    ZeroMemory( &msg, sizeof(msg) );
    while( msg.message!=WM_QUIT )
    {
        if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
        {
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
        else 
		{
            Render();
		}
    }
   
	// The program is now finishing so exit gracefully

	// Must clean up DirectX first
	CleanupDirectX();

	return msg.wParam;
}

/**************************************************************************************************
Desc: To initialise DirectX we must create the Direct3D object and then a device
**************************************************************************************************/
bool InitialiseDirectX(HWND hWnd)
{	
	gD3dObject=Direct3DCreate9(D3D_SDK_VERSION);
	if (gD3dObject==NULL)
		return FALSE;

	D3DPRESENT_PARAMETERS presParams;
	ZeroMemory(&presParams,sizeof(presParams));

	presParams.Windowed=TRUE;
	presParams.SwapEffect=D3DSWAPEFFECT_DISCARD;
	presParams.BackBufferFormat=D3DFMT_UNKNOWN;
	presParams.BackBufferCount=1;

	// I would prefer to use hardware processing (D3DCREATE_HARDWARE_VERTEXPROCESSING) but so as to 
	// support older cards I set this to software (D3DCREATE_SOFTWARE_VERTEXPROCESSING)
	HRESULT hr=gD3dObject->CreateDevice(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING, &presParams, &gD3dDevice);

	if (FAILED(hr))
		return FALSE;	

	return TRUE;
}

/**************************************************************************************************
Desc: When we created our window class we told windows to call this function whenever a message is 
recieved directed at this window. We did this by providing a pointer to this function in the window class structure
We can either trap the messages ourselves or let Windows handle the message in a default way by passing it to DefWindowProc
**************************************************************************************************/
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{	
	switch (message) 
	{
		// Handle menu messages
		case WM_COMMAND:
		{
			int wmId    = LOWORD(wParam); 
			int wmEvent = HIWORD(wParam); 

			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_EXIT:
				   DestroyWindow(hWnd);
				break;
			}
		}
		break;

		// Handle destroy message
		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		// All other messages get passed back to Windows to handle in a default way
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

/**************************************************************************************************
Desc: In response to windows requiring our window to be redrawn we use Direct3D to display our scene
**************************************************************************************************/
void Render()
{	
	// 1. clear the back buffer
	HRESULT hr=gD3dDevice->Clear(0,NULL,D3DCLEAR_TARGET,D3DCOLOR_XRGB(0,0,0),1.0f,0);

	// 2. begin the scene
	hr=gD3dDevice->BeginScene();
	if (SUCCEEDED(hr))
	{
		RECT rct;
		rct.left=0;
		rct.right=gTextureWidth;	    

		D3DXVECTOR3 pos(0,0,0);

		// Draw our sprites
		gSprite->Begin(0);		

		// Top sprite		
		rct.top=gTextureHeight-gOffset;
		rct.bottom=gTextureHeight;
		
		gSprite->Draw(gTexture, &rct, NULL,&pos,0xFFFFFFFF);

		// Bottom sprite
		rct.top=0;
		rct.bottom=gScreenHeight-gOffset;

		pos.y=(float)gOffset;

		gSprite->Draw(gTexture, &rct, NULL,&pos,0xFFFFFFFF);

		// Done
		gSprite->End();

		gD3dDevice->EndScene();
		gD3dDevice->Present(NULL,NULL,NULL,NULL);
	}	

	// Increment our offset and loop if required
	gOffset++;
	if (gOffset>gTextureHeight)
		gOffset=0;
}

/**************************************************************************************************
Desc: Our application is finishing so we must clean up by deallocating any DirectX objects
**************************************************************************************************/
void CleanupDirectX()
{
	if (gTexture)
	{
		gTexture->Release();
		gTexture=NULL;
	}

	if (gSprite)
	{
		gSprite->Release();
		gSprite=NULL;
	}

	if (gD3dObject)
	{
		gD3dObject->Release();
		gD3dObject=NULL;
	}

	if (gD3dDevice)
	{
		gD3dDevice->Release();
		gD3dDevice=NULL;
	}
}

/**************************************************************************************************
Desc: Create our sprite. Return FALSE if an error occured.
**************************************************************************************************/
bool CreateSprite()
{
	// Create our sprite object - only one needed as it is just a set of functions
	if (FAILED(D3DXCreateSprite(gD3dDevice,&gSprite)))
		return FALSE;

	// Load a texture for our sprite
	if (FAILED(D3DXCreateTextureFromFile(gD3dDevice,"suburbs.dds",&gTexture)))
		return FALSE;

	// Get the dimensions of our txture
	D3DSURFACE_DESC desc;
	gTexture->GetLevelDesc(0,&desc);

	gTextureWidth=desc.Width;
	gTextureHeight=desc.Height;

	return TRUE;
}

